@@ -2,9 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var Ci = Components.interfaces;
-var Cc = Components.classes;
-var Cu = Components.utils;
+const MODULE_NAME = "window-helpers";
+
+Cu.import('resource:///modules/iteratorUtils.jsm');
+Cu.import('resource://gre/modules/NetUtil.jsm');
+Cu.import("resource://gre/modules/Services.jsm");
 
 var mozmill = {};
 Cu.import('resource://mozmill/modules/mozmill.js', mozmill);
@@ -17,12 +19,6 @@ Cu.import('resource://mozmill/modules/frame.js', frame);
 var utils = {};
 Cu.import('resource://mozmill/modules/utils.js', utils);
 
-Cu.import('resource:///modules/iteratorUtils.jsm');
-Cu.import('resource://gre/modules/NetUtil.jsm');
-Cu.import("resource://gre/modules/Services.jsm");
-
-const MODULE_NAME = 'window-helpers';
-
 /**
  * Timeout to use when waiting for the first window ever to load.  This is
  *  long because we are basically waiting for the entire app startup process.
@@ -76,10 +72,6 @@ function hereIsMarkAction(mark_action_impl, mark_failure_impl,
   normalize_for_json = normalize_for_json_impl;
 }
 
-function setupModule() {
-  // do nothing
-}
-
 function installInto(module) {
   module.plan_for_new_window = plan_for_new_window;
   module.wait_for_new_window = wait_for_new_window;
@@ -89,6 +81,7 @@ function installInto(module) {
   module.wait_for_window_close = wait_for_window_close;
   module.close_window = close_window;
   module.wait_for_existing_window = wait_for_existing_window;
+  module.wait_for_window_focused = wait_for_window_focused;
 
   module.wait_for_browser_load = wait_for_browser_load;
   module.wait_for_frame_load = wait_for_frame_load;
@@ -209,14 +202,14 @@ var WindowWatcher = {
    *  that appear and dis-appear do so without dangerously confusing us (as
    *  long as another one comes along...)
    */
-  waitingList: {},
+  waitingList: new Map(),
   /**
    * Note that we will be looking for a window with the given window type
    *  (ex: "mailnews:search").  This allows us to be ready if an event shows
    *  up before waitForWindow is called.
    */
   planForWindowOpen: function WindowWatcher_planForWindowOpen(aWindowType) {
-    this.waitingList[aWindowType] = null;
+    this.waitingList.set(aWindowType, null);
   },
 
   /**
@@ -224,7 +217,7 @@ var WindowWatcher = {
    */
   planForAlreadyOpenWindow:
       function WindowWatcher_planForAlreadyOpenWindow(aWindowType) {
-    this.waitingList[aWindowType] = null;
+    this.waitingList.set(aWindowType, null);
     // We need to iterate over all the XUL windows and consider them all.
     //  We can't pass the window type because the window might not have a
     //  window type yet.
@@ -260,10 +253,10 @@ var WindowWatcher = {
                   this);
 
     this.waitingForOpen = null;
-    let xulWindow = this.waitingList[aWindowType];
+    let xulWindow = this.waitingList.get(aWindowType);
     let domWindow = xulWindow.docShell.QueryInterface(Ci.nsIInterfaceRequestor)
                                       .getInterface(Ci.nsIDOMWindow);
-    delete this.waitingList[aWindowType];
+    this.waitingList.delete(aWindowType);
     // spin the event loop to make sure any setTimeout 0 calls have gotten their
     //  time in the sun.
     controller.sleep(0);
@@ -296,7 +289,7 @@ var WindowWatcher = {
       this._timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
     this.waitingForOpen = aWindowType;
     this.subTestFunc = aSubTestFunc;
-    this.waitingList[aWindowType] = null;
+    this.waitingList.set(aWindowType, null);
 
     this._timerRuntimeSoFar = 0;
     this._timer.initWithCallback(this, WINDOW_OPEN_CHECK_INTERVAL_MS,
@@ -309,31 +302,58 @@ var WindowWatcher = {
   notify: function WindowWatcher_notify() {
     if (this.monitorizeOpen()) {
       // okay, the window is opened, and we should be in its event loop now.
-      let xulWindow = this.waitingList[this.waitingForOpen];
+      let xulWindow = this.waitingList.get(this.waitingForOpen);
       let domWindow = xulWindow.docShell.QueryInterface(Ci.nsIInterfaceRequestor)
                                         .getInterface(Ci.nsIDOMWindow);
       let troller = new controller.MozMillController(domWindow);
       augment_controller(troller, this.waitingForOpen);
 
-      delete this.waitingList[this.waitingForOpen];
       this._timer.cancel();
 
-      // now we are waiting for it to close...
-      this.waitingForClose = this.waitingForOpen;
-      this.waitingForOpen = null;
+      let self = this;
+      function startTest() {
+        try {
+          let runner = new frame.Runner(collector);
+          runner.wrapper(self.subTestFunc, troller);
+        }
+        finally {
+          self.subTestFunc = null;
+        }
 
-      try {
-        let runner = new frame.Runner(collector);
-        runner.wrapper(this.subTestFunc, troller);
-      }
-      finally {
-        this.subTestFunc = null;
+        // if the test failed, make sure we force the window closed...
+        // except I'm not sure how to easily figure that out...
+        // so just close it no matter what.
+        troller.window.close();
+
+        self.waitingList.delete(self.waitingForOpen);
+        // now we are waiting for it to close...
+        self.waitingForClose = self.waitingForOpen;
+        self.waitingForOpen = null;
       }
 
-      // if the test failed, make sure we force the window closed...
-      // except I'm not sure how to easily figure that out...
-      // so just close it no matter what.
-      troller.window.close();
+      let targetFocusedWindow = {};
+      Services.focus.getFocusedElementForWindow(domWindow, true,
+                                                targetFocusedWindow);
+      targetFocusedWindow = targetFocusedWindow.value;
+
+      let focusedWindow = {};
+      if (Services.focus.activeWindow) {
+        Services.focus.getFocusedElementForWindow(Services.focus.activeWindow, true,
+                                                  focusedWindow);
+
+        focusedWindow = focusedWindow.value;
+      }
+
+      if (focusedWindow == targetFocusedWindow) {
+        startTest();
+      } else {
+        function onFocus(event) {
+          targetFocusedWindow.removeEventListener("focus", onFocus, true);
+          targetFocusedWindow.setTimeout(startTest, 0);
+        }
+        targetFocusedWindow.addEventListener("focus", onFocus, true);
+        targetFocusedWindow.focus();
+      }
     }
     // notify is only used for modal dialogs, which are never the first window,
     //  so we can always just use this set of timeouts/intervals.
@@ -365,7 +385,7 @@ var WindowWatcher = {
     let windowType =
       aXULWindow.document.documentElement.getAttribute("windowtype") ||
       aXULWindow.document.documentElement.getAttribute("id");
-    this.waitingList[windowType] = aXULWindow;
+    this.waitingList.set(windowType, aXULWindow);
     this.waitingForClose = windowType;
   },
 
@@ -378,8 +398,8 @@ var WindowWatcher = {
     utils.waitFor(function () this.monitorizeClose(),
                   "Timeout waiting for window to close!",
       WINDOW_CLOSE_TIMEOUT_MS, WINDOW_CLOSE_CHECK_INTERVAL_MS, this);
-    let didDisappear = this.waitingList[this.waitingForClose] == null;
-    delete this.waitingList[windowType];
+    let didDisappear = (this.waitingList.get(this.waitingForClose) == null);
+    this.waitingList.delete(windowType);
     let windowType = this.waitingForClose;
     this.waitingForClose = null;
     if (!didDisappear)
@@ -412,7 +432,8 @@ var WindowWatcher = {
         this.monitoringList.splice(iWin, 1);
     }
 
-    return this.waitingList[this.waitingForOpen] != null;
+    return this.waitingList.has(this.waitingForOpen) &&
+           (this.waitingList.get(this.waitingForOpen) != null);
   },
 
   /**
@@ -422,7 +443,7 @@ var WindowWatcher = {
    * @return true if it closed.
    */
   monitorizeClose: function () {
-    return this.waitingList[this.waitingForClose] == null;
+    return this.waitingList.get(this.waitingForClose) == null;
   },
 
   /**
@@ -475,8 +496,8 @@ var WindowWatcher = {
       return false;
 
     // stash the window if we were watching for it
-    if (windowType in this.waitingList) {
-      this.waitingList[windowType] = aXULWindow;
+    if (this.waitingList.has(windowType)) {
+      this.waitingList.set(windowType, aXULWindow);
     }
 
     return true;
@@ -495,10 +516,8 @@ var WindowWatcher = {
     mark_action("winhelp", "onCloseWindow",
                 [getWindowTypeForXulWindow(aXULWindow, true) +
                    " (" + getUniqueIdForXulWindow(aXULWindow) + ")"]);
-    // XXX because of how we dance with things, equivalence is not gonna
-    //  happen for us.  This is most pragmatic.
-    if (this.waitingList[windowType] !== null)
-      this.waitingList[windowType] = null;
+    if (this.waitingList.has(windowType))
+      this.waitingList.set(windowType, null);
   },
 };
 
@@ -630,6 +649,42 @@ function close_window(aController) {
   wait_for_window_close();
 }
 
+/**
+ * Wait for the window to be focused.
+ *
+ * @param aWindow the window to be focused.
+ */
+function wait_for_window_focused(aWindow) {
+  let targetWindow = {};
+
+  Services.focus.getFocusedElementForWindow(aWindow, true,
+                                            targetWindow);
+  targetWindow = targetWindow.value;
+
+  let focusedWindow = {};
+  if (Services.focus.activeWindow) {
+    Services.focus.getFocusedElementForWindow(Services.focus.activeWindow,
+                                              true, focusedWindow);
+    focusedWindow = focusedWindow.value;
+  }
+
+  let focused = false;
+  if (focusedWindow == targetWindow) {
+    focused = true;
+  } else {
+    function onFocus(event) {
+      targetWindow.removeEventListener("focus", onFocus, true);
+      focused = true;
+    }
+    targetWindow.addEventListener("focus", onFocus, true);
+    targetWindow.focus();
+  }
+
+  utils.waitFor(function() focused,
+      "Timeout waiting for window to be focused.",
+      WINDOW_FOCUS_TIMEOUT_MS, 100, this);
+}
+
 /**
  * Given a <browser>, waits for it to completely load.
  *
@@ -892,17 +947,22 @@ var AugmentEverybodyWith = {
      *     attribute with a single value.  We pick the menu option whose DOM
      *     node has an attribute with that name and value.  We click whatever we
      *     find.  We throw if we don't find what you were asking for.
+     * @param aKeepOpen  If set to true the popups are not closed after last click.
+     *
+     * @return  An array of popup elements that were left open. It will be
+     *          an empty array if aKeepOpen was set to true.
      */
-    click_menus_in_sequence: function _click_menus(aRootPopup, aActions) {
+    click_menus_in_sequence: function _click_menus(aRootPopup, aActions, aKeepOpen) {
       if (aRootPopup.state == "closed")
         aRootPopup.openPopup(null, "", 0, 0, true, true);
+      aRootPopup.focus(); // This is a hack that can be removed once the focus issues on Linux are solved.
       if (aRootPopup.state != "open") { // handle "showing"
         utils.waitFor(function() { return aRootPopup.state == "open"; },
                       "Popup never opened! id=" + aRootPopup.id +
                       ", state=" + aRootPopup.state, 5000, 50);
       }
       // These popups sadly do not close themselves, so we need to keep track
-      //  of them so we can make sure they end up closed.
+      // of them so we can make sure they end up closed.
       let closeStack = [aRootPopup];
 
       let curPopup = aRootPopup;
@@ -932,6 +992,7 @@ var AugmentEverybodyWith = {
                           iAction);
 
         this.click(new elib.Elem(matchingNode));
+        matchingNode.focus(); // This is a hack that can be removed once the focus issues on Linux are solved.
         if ("menupopup" in matchingNode) {
           curPopup = matchingNode.menupopup;
           closeStack.push(curPopup);
@@ -942,8 +1003,24 @@ var AugmentEverybodyWith = {
         }
       }
 
-      while (closeStack.length) {
-        curPopup = closeStack.pop();
+      if (!aKeepOpen) {
+        this.close_popup_sequence(closeStack);
+        return [];
+      } else {
+        return closeStack;
+      }
+    },
+
+    /**
+     * Close given menupopups.
+     *
+     * @param aCloseStack  An array of menupopup elements that are to be closed.
+     *                     The elements are processed from the end of the array
+     *                     to the front (a stack).
+     */
+    close_popup_sequence: function _close_popup_sequence(aCloseStack) {
+      while (aCloseStack.length) {
+        let curPopup = aCloseStack.pop();
         this.keypress(new elib.Elem(curPopup), "VK_ESCAPE", {});
         utils.waitFor(function() { return curPopup.state == "closed"; },
                       "Popup did not close! id=" + curPopup.id +
@@ -1333,7 +1410,7 @@ populateDomKeycodeMap();
 /**
  * Given something you would find on event.target (should be a DOM node /
  *  DOM window), attempt to describe the hierarchy of that thing all the way
- *  to the outermost enclosing window.  This is intended to solve the probem
+ *  to the outermost enclosing window.  This is intended to solve the problem
  *  where our event target can be the "Window" of an iframe, which is not
  *  very enlightening.  We really want to know the frameElement and what
  *  window it lives in.
@@ -1348,20 +1425,24 @@ populateDomKeycodeMap();
  * @returns a list suitable for concatenating with another list to be passed
  *   to mark_action.
  */
-function describeEventElementInHierarchy(elem) {
-  let arr = [], win;
-  // DOM element.
-  if ("ownerDocument" in elem) {
-    arr.push(normalize_for_json(elem));
-    win = elem.ownerDocument.defaultView;
+function describeEventElementInHierarchy(aNode) {
+  let arr = [];
+  let win = null;
+  // DOM node ?
+  if ("ownerDocument" in aNode) {
+    arr.push(normalize_for_json(aNode));
+    if (aNode.ownerDocument)
+      win = aNode.ownerDocument.defaultView;
+    else
+      win = aNode.defaultView;
   }
-  // should already be a window
   else {
-    win = elem;
+    // Otherwise this should be a window.
+    win = aNode;
   }
   let treeItem = win.QueryInterface(Ci.nsIInterfaceRequestor)
-                  .getInterface(Ci.nsIWebNavigation)
-                  .QueryInterface(Ci.nsIDocShellTreeItem);
+                    .getInterface(Ci.nsIWebNavigation)
+                    .QueryInterface(Ci.nsIDocShellTreeItem);
   while (treeItem) {
     win = treeItem.QueryInterface(Ci.nsIInterfaceRequestor)
                   .getInterface(Ci.nsIDOMWindow);
