@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Eric Ballet Baz BT Global Services / Etat francais Ministere de la Defense
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #include "nsCOMPtr.h"
 #include "nsMsgCompUtils.h"
 #include "nsIPrefService.h"
@@ -140,16 +107,16 @@ nsMsgCreateTempFileName(const char *tFileName)
                                                 tFileName,
                                                 getter_AddRefs(tmpFile));
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   rv = tmpFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 00600);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   nsCString tempString;
   rv = tmpFile->GetNativePath(tempString);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   char *tString = ToNewCString(tempString);
   if (!tString)
@@ -189,6 +156,38 @@ nsMsgMIMESetConformToStandard (bool conform_p)
   }
 }
 
+/**
+ * Checks if the recipient fields have sane values for message send.
+ */
+nsresult mime_sanity_check_fields_recipients (
+          const char *to,
+          const char *cc,
+          const char *bcc,
+          const char *newsgroups)
+{
+  if (to)
+    while (IS_SPACE(*to))
+      to++;
+  if (cc)
+    while (IS_SPACE(*cc))
+      cc++;
+  if (bcc)
+    while (IS_SPACE(*bcc))
+      bcc++;
+  if (newsgroups)
+    while (IS_SPACE(*newsgroups))
+      newsgroups++;
+
+  if ((!to || !*to) && (!cc || !*cc) &&
+      (!bcc || !*bcc) && (!newsgroups || !*newsgroups))
+    return NS_MSG_NO_RECIPIENTS;
+
+  return NS_OK;
+}
+
+/**
+ * Checks if the compose fields have sane values for message send.
+ */
 nsresult mime_sanity_check_fields (
           const char *from,
           const char *reply_to,
@@ -204,39 +203,23 @@ nsresult mime_sanity_check_fields (
           const char * /*other_random_headers*/)
 {
   if (from)
-    while (IS_SPACE (*from))
+    while (IS_SPACE(*from))
       from++;
   if (reply_to)
-    while (IS_SPACE (*reply_to))
+    while (IS_SPACE(*reply_to))
       reply_to++;
-  if (to)
-    while (IS_SPACE (*to))
-      to++;
-  if (cc)
-    while (IS_SPACE (*cc))
-      cc++;
-  if (bcc)
-    while (IS_SPACE (*bcc))
-      bcc++;
   if (fcc)
-    while (IS_SPACE (*fcc))
+    while (IS_SPACE(*fcc))
       fcc++;
-  if (newsgroups)
-    while (IS_SPACE (*newsgroups))
-      newsgroups++;
   if (followup_to)
-    while (IS_SPACE (*followup_to))
+    while (IS_SPACE(*followup_to))
       followup_to++;
 
-  /* #### sanity check other_random_headers for newline conventions */
+  // TODO: sanity check other_random_headers for newline conventions
   if (!from || !*from)
     return NS_MSG_NO_SENDER;
-  else
-    if ((!to || !*to) && (!cc || !*cc) &&
-        (!bcc || !*bcc) && (!newsgroups || !*newsgroups))
-      return NS_MSG_NO_RECIPIENTS;
-  else
-    return NS_OK;
+
+  return mime_sanity_check_fields_recipients(to, cc, bcc, newsgroups);
 }
 
 static char *
@@ -284,20 +267,20 @@ nsMsgStripLine (char * string)
 char *
 mime_generate_headers (nsMsgCompFields *fields,
                        const char *charset,
-                       nsMsgDeliverMode deliver_mode, nsIPrompt * aPrompt, PRInt32 *status)
+                       nsMsgDeliverMode deliver_mode, nsIPrompt * aPrompt, nsresult *status)
 {
   nsresult rv;
-  *status = 0;
+  *status = NS_OK;
 
   nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
   if (NS_FAILED(rv)) {
     *status = rv;
-    return nsnull;
+    return nullptr;
   }
 
   bool usemime = nsMsgMIMEGetConformToStandard();
-  PRInt32 size = 0;
-  char *buffer = 0, *buffer_tail = 0;
+  int32_t size = 0;
+  char *buffer = nullptr, *buffer_tail = nullptr;
   bool isDraft =
     deliver_mode == nsIMsgSend::nsMsgSaveAsDraft ||
     deliver_mode == nsIMsgSend::nsMsgSaveAsTemplate ||
@@ -320,13 +303,14 @@ mime_generate_headers (nsMsgCompFields *fields,
 
   bool hasDisclosedRecipient = false;
 
-  nsCAutoString headerBuf;    // accumulate header strings to get length
+  nsAutoCString headerBuf;    // accumulate header strings to get length
   headerBuf.Truncate();
 
   NS_ASSERTION (fields, "null fields");
-  if (!fields)
-    return nsnull;
-
+  if (!fields) {
+    *status = NS_ERROR_NULL_POINTER;
+    return nullptr;
+  }
   pFrom = fields->GetFrom();
   if (pFrom)
     headerBuf.Append(pFrom);
@@ -360,8 +344,10 @@ mime_generate_headers (nsMsgCompFields *fields,
   size += 2560;
 
   buffer = (char *) PR_Malloc (size);
-  if (!buffer)
-    return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
+  if (!buffer) {
+    *status = NS_ERROR_OUT_OF_MEMORY;
+    return nullptr; /* NS_ERROR_OUT_OF_MEMORY */
+  }
 
   buffer_tail = buffer;
 
@@ -379,7 +365,7 @@ mime_generate_headers (nsMsgCompFields *fields,
       (deliver_mode != nsIMsgSend::nsMsgSaveAsDraft &&
       deliver_mode != nsIMsgSend::nsMsgSaveAsTemplate))
     {
-        PRInt32 receipt_header_type = nsIMsgMdnGenerator::eDntType;
+        int32_t receipt_header_type = nsIMsgMdnGenerator::eDntType;
         fields->GetReceiptHeaderType(&receipt_header_type);
 
       // nsIMsgMdnGenerator::eDntType = MDN Disposition-Notification-To: ;
@@ -453,7 +439,7 @@ mime_generate_headers (nsMsgCompFields *fields,
       // whether the draft/template has request for either MDN or DNS or both
       // return receipt; since the DNS is out of the picture we now use the
       // header type + 1 to tell whether user has requested the return receipt
-      PRInt32 headerType = 0;
+      int32_t headerType = 0;
       fields->GetReceiptHeaderType(&headerType);
       char *type = PR_smprintf("%d", (int)headerType + 1);
       if (type)
@@ -480,7 +466,7 @@ mime_generate_headers (nsMsgCompFields *fields,
   nsCOMPtr<nsIHttpProtocolHandler> pHTTPHandler = do_GetService(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX "http", &rv);
   if (NS_SUCCEEDED(rv) && pHTTPHandler)
   {
-    nsCAutoString userAgentString;
+    nsAutoCString userAgentString;
     pHTTPHandler->GetUserAgent(userAgentString);
 
     if (!userAgentString.IsEmpty())
@@ -499,7 +485,8 @@ mime_generate_headers (nsMsgCompFields *fields,
     char *duppedNewsGrp = PL_strdup(pNewsGrp);
     if (!duppedNewsGrp) {
       PR_FREEIF(buffer);
-      return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
+      *status = NS_ERROR_OUT_OF_MEMORY;
+      return nullptr; /* NS_ERROR_OUT_OF_MEMORY */
     }
     char *n2 = nsMsgStripLine(duppedNewsGrp);
 
@@ -532,10 +519,10 @@ mime_generate_headers (nsMsgCompFields *fields,
     // "news://news.mozilla.org/netscape.test,news://news.mozilla.org/netscape.junk"
     // we need to turn that into: "netscape.test,netscape.junk"
     //
-    nsCOMPtr <nsINntpService> nntpService = do_GetService("@mozilla.org/messenger/nntpservice;1");
+    nsCOMPtr<nsINntpService> nntpService = do_GetService("@mozilla.org/messenger/nntpservice;1", &rv);
     if (NS_FAILED(rv) || !nntpService) {
       *status = NS_ERROR_FAILURE;
-      return nsnull;
+      return nullptr;
     }
 
     nsCString newsgroupsHeaderVal;
@@ -544,7 +531,7 @@ mime_generate_headers (nsMsgCompFields *fields,
     if (NS_FAILED(rv)) 
     {
       *status = rv;
-      return nsnull;
+      return nullptr;
     }
 
     // fixme:the newsgroups header had better be encoded as the server-side
@@ -578,7 +565,7 @@ mime_generate_headers (nsMsgCompFields *fields,
     char *duppedFollowup = PL_strdup(pFollow);
     if (!duppedFollowup) {
       PR_FREEIF(buffer);
-      return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
+      return nullptr; /* NS_ERROR_OUT_OF_MEMORY */
     }
     char *n2 = nsMsgStripLine (duppedFollowup);
 
@@ -665,8 +652,8 @@ mime_generate_headers (nsMsgCompFields *fields,
 
     // Skip default priority.
     if (priorityValue != nsMsgPriority::Default) {
-      nsCAutoString priorityName;
-      nsCAutoString priorityValueString;
+      nsAutoCString priorityName;
+      nsAutoCString priorityValueString;
 
       NS_MsgGetPriorityValueString(priorityValue, priorityValueString);
       NS_MsgGetUntranslatedPriorityName(priorityValue, priorityName);
@@ -724,17 +711,20 @@ mime_generate_headers (nsMsgCompFields *fields,
     PUSH_STRING (pOtherHdr);
   }
 
-  if (buffer_tail > buffer + size - 1)
-    return nsnull;
-
+  if (buffer_tail > buffer + size - 1) {
+    PR_FREEIF(buffer);
+    return nullptr;
+  }
   /* realloc it smaller... */
-  buffer = (char*)PR_REALLOC (buffer, buffer_tail - buffer + 1);
+  char *newBuffer = (char*) PR_REALLOC(buffer, buffer_tail - buffer + 1);
+  if (!newBuffer) // The original bigger buffer is still usable to the caller.
+    return buffer;
 
-  return buffer;
+  return newBuffer;
 }
 
 static void
-GenerateGlobalRandomBytes(unsigned char *buf, PRInt32 len)
+GenerateGlobalRandomBytes(unsigned char *buf, int32_t len)
 {
   static bool      firstTime = true;
 
@@ -742,13 +732,11 @@ GenerateGlobalRandomBytes(unsigned char *buf, PRInt32 len)
   {
     // Seed the random-number generator with current time so that
     // the numbers will be different every time we run.
-    PRInt32 aTime;
-    LL_L2I(aTime, PR_Now());
-    srand( (unsigned)aTime );
+    srand( (unsigned)PR_Now() );
     firstTime = false;
   }
 
-  for( PRInt32 i = 0; i < len; i++ )
+  for( int32_t i = 0; i < len; i++ )
     buf[i] = rand() % 10;
 }
 
@@ -774,7 +762,7 @@ RFC2231ParmFolding(const char *parmName, const nsCString& charset,
 
 static char *
 LegacyParmFolding(const nsCString& aCharset,
-                  const nsCString& aFileName, PRInt32 aParmFolding);
+                  const nsCString& aFileName, int32_t aParmFolding);
 
 char *
 mime_generate_attachment_headers (const char *type,
@@ -795,13 +783,13 @@ mime_generate_attachment_headers (const char *type,
 {
   NS_ASSERTION (encoding, "null encoding");
 
-  PRInt32 parmFolding = 2; // RFC 2231-compliant
+  int32_t parmFolding = 2; // RFC 2231-compliant
   nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
   if (prefs)
     prefs->GetIntPref("mail.strictly_mime.parm_folding", &parmFolding);
 
   /* Let's encode the real name */
-  char *encodedRealName = nsnull;
+  char *encodedRealName = nullptr;
   nsCString charset;   // actual charset used for MIME encode
   nsAutoString realName;
   if (real_name)
@@ -819,7 +807,7 @@ mime_generate_attachment_headers (const char *type,
         charset.Assign("UTF-8"); // set to UTF-8 if fails again
     }
 
-    encodedRealName = RFC2231ParmFolding("filename", charset, nsnull,
+    encodedRealName = RFC2231ParmFolding("filename", charset, nullptr,
                                          realName);
     // somehow RFC2231ParamFolding failed. fall back to legacy method
     if (!encodedRealName || !*encodedRealName) {
@@ -964,7 +952,7 @@ mime_generate_attachment_headers (const char *type,
 
   if (encodedRealName && *encodedRealName) {
     char *period = PL_strrchr(encodedRealName, '.');
-    PRInt32 pref_content_disposition = 0;
+    int32_t pref_content_disposition = 0;
 
     if (prefs) {
       nsresult rv = prefs->GetIntPref("mail.content_disposition_type",
@@ -1014,7 +1002,7 @@ mime_generate_attachment_headers (const char *type,
       !PL_strcasecmp (type, TEXT_MDL)) &&
       base_url && *base_url)
   {
-    PRInt32 col = 0;
+    int32_t col = 0;
     const char *s = base_url;
     const char *colon = PL_strchr (s, ':');
     bool useContentLocation = false;   /* rhp - add this  */
@@ -1048,7 +1036,7 @@ CONTENT_LOC_HACK:
 
     while (*s != 0 && *s != '#')
     {
-      PRUint32 ot=buf.Length();
+      uint32_t ot=buf.Length();
       char tmp[]="\x00\x00";
       /* URLs must be wrapped at 40 characters or less. */
       if (col >= 38) {
@@ -1108,25 +1096,19 @@ static bool isValidHost( const char* host )
          && *s != '.'
          )
       {
-       host = nsnull;
+       host = nullptr;
        break;
       }
 
-  return nsnull != host;
+  return nullptr != host;
 }
 
 char *
 msg_generate_message_id (nsIMsgIdentity *identity)
 {
-  PRUint32 now;
-  PRTime prNow = PR_Now();
-  PRInt64 microSecondsPerSecond, intermediateResult;
+  uint32_t now = (uint32_t)(PR_Now() / PR_USEC_PER_SEC);
 
-  LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
-  LL_DIV(intermediateResult, prNow, microSecondsPerSecond);
-    LL_L2UI(now, intermediateResult);
-
-  PRUint32 salt = 0;
+  uint32_t salt = 0;
   const char *host = 0;
 
   nsCString forcedFQDN;
@@ -1174,14 +1156,14 @@ inline static bool is7bitCharset(const nsCString& charset)
 RFC2231ParmFolding(const char *parmName, const nsCString& charset,
                    const char *language, const nsString& parmValue)
 {
-  NS_ENSURE_TRUE(parmName && *parmName && !parmValue.IsEmpty(), nsnull);
+  NS_ENSURE_TRUE(parmName && *parmName && !parmValue.IsEmpty(), nullptr);
 
   bool needEscape;
   nsCString dupParm;
 
   if (!NS_IsAscii(parmValue.get()) || is7bitCharset(charset)) {
     needEscape = true;
-    nsCAutoString nativeParmValue;
+    nsAutoCString nativeParmValue;
     ConvertFromUnicode(charset.get(), parmValue, nativeParmValue);
     MsgEscapeString(nativeParmValue, nsINetUtil::ESCAPE_ALL, dupParm);
   }
@@ -1193,19 +1175,19 @@ RFC2231ParmFolding(const char *parmName, const nsCString& charset,
   }
 
   if (dupParm.IsEmpty())
-    return nsnull;
+    return nullptr;
 
-  PRInt32 parmNameLen = PL_strlen(parmName);
-  PRInt32 parmValueLen = dupParm.Length();
+  int32_t parmNameLen = PL_strlen(parmName);
+  int32_t parmValueLen = dupParm.Length();
 
   if (needEscape)
     parmNameLen += 5;   // *=__'__'___ or *[0]*=__'__'__ or *[1]*=___
   else
     parmNameLen += 5;   // *[0]="___";
 
-  PRInt32 languageLen = language ?  PL_strlen(language) : 0;
-  PRInt32 charsetLen = charset.Length();
-  char *foldedParm = nsnull;
+  int32_t languageLen = language ?  PL_strlen(language) : 0;
+  int32_t charsetLen = charset.Length();
+  char *foldedParm = nullptr;
 
   if ((parmValueLen + parmNameLen + charsetLen + languageLen) <
       PR_MAX_FOLDING_LEN)
@@ -1282,25 +1264,25 @@ RFC2231ParmFolding(const char *parmName, const nsCString& charset,
         // check to see if we are in the middle of escaped char
         if (*end == '%')
         {
-          tmp = '%'; *end = nsnull;
+          tmp = '%'; *end = 0;
         }
         else if (end-1 > start && *(end-1) == '%')
         {
-          end -= 1; tmp = '%'; *end = nsnull;
+          end -= 1; tmp = '%'; *end = 0;
         }
         else if (end-2 > start && *(end-2) == '%')
         {
-          end -= 2; tmp = '%'; *end = nsnull;
+          end -= 2; tmp = '%'; *end = 0;
         }
         else
         {
-          tmp = *end; *end = nsnull;
+          tmp = *end; *end = 0;
         }
       }
       else
       {
         // XXX should check if we are in the middle of escaped char (RFC 822)
-        tmp = *end; *end = nsnull;
+        tmp = *end; *end = 0;
       }
       NS_MsgSACat(&foldedParm, start);
       if (!needEscape)
@@ -1318,7 +1300,7 @@ RFC2231ParmFolding(const char *parmName, const nsCString& charset,
 
 /*static */ char *
 LegacyParmFolding(const nsCString& aCharset,
-                  const nsCString& aFileName, PRInt32 aParmFolding)
+                  const nsCString& aFileName, int32_t aParmFolding)
 {
   bool usemime = nsMsgMIMEGetConformToStandard();
   char *encodedRealName =
@@ -1343,13 +1325,13 @@ LegacyParmFolding(const nsCString& aCharset,
 }
 
 bool
-mime_7bit_data_p (const char *string, PRUint32 size)
+mime_7bit_data_p (const char *string, uint32_t size)
 {
   if ((!string) || (!*string))
     return true;
 
   char *ptr = (char *)string;
-  for (PRUint32 i=0; i<size; i++)
+  for (uint32_t i=0; i<size; i++)
   {
     if ((unsigned char) ptr[i] > 0x7F)
       return false;
@@ -1368,7 +1350,7 @@ mime_fix_header_1 (const char *string, bool addr_p, bool news_p)
   char *new_string;
   const char *in;
   char *out;
-  PRInt32 i, old_size, new_size;
+  int32_t i, old_size, new_size;
 
   if (!string || !*string)
     return 0;
@@ -1633,15 +1615,15 @@ msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *propose
     if (StringBeginsWith(url, NS_LITERAL_CSTRING("data:"),
                          nsCaseInsensitiveCStringComparator()))
     {
-      PRInt32 endNonData = url.FindChar(',');
+      int32_t endNonData = url.FindChar(',');
       if (endNonData == -1)
         return;
       nsCString nonDataPart(Substring(url, 5, endNonData - 5));
-      PRInt32 filenamePos = nonDataPart.Find("filename=");
+      int32_t filenamePos = nonDataPart.Find("filename=");
       if (filenamePos != -1)
       {
         filenamePos += 9;
-        PRInt32 endFilename = nonDataPart.FindChar(';', filenamePos);
+        int32_t endFilename = nonDataPart.FindChar(';', filenamePos);
         if (endFilename == -1)
           endFilename = endNonData;
         attachment->m_realName = Substring(nonDataPart, filenamePos,
@@ -1663,7 +1645,7 @@ msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *propose
         mimeInfo->GetPrimaryExtension(extension);
         unsigned char filePrefix[10];
         GenerateGlobalRandomBytes(filePrefix, 8);
-        for (PRInt32 i = 0; i < 8; i++)
+        for (int32_t i = 0; i < 8; i++)
           filename.Append(filePrefix[i] + 'a');
         filename.Append('.');
         filename.Append(extension);
@@ -1680,7 +1662,7 @@ msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *propose
       if (s2) s = s2+1;
       /* Copy it into the attachment struct. */
       attachment->m_realName = s;
-      PRInt32 charPos = attachment->m_realName.FindChar('?');
+      int32_t charPos = attachment->m_realName.FindChar('?');
       if (charPos != -1)
         attachment->m_realName.SetLength(charPos);
       /* Now trim off any named anchors or search data. */
@@ -1739,20 +1721,20 @@ nsresult
 nsMsgNewURL(nsIURI** aInstancePtrResult, const char * aSpec)
 {
   nsresult rv = NS_OK;
-  if (nsnull == aInstancePtrResult)
+  if (nullptr == aInstancePtrResult)
     return NS_ERROR_NULL_POINTER;
   nsCOMPtr<nsIIOService> pNetService =
     mozilla::services::GetIOService();
   NS_ENSURE_TRUE(pNetService, NS_ERROR_UNEXPECTED);
-  if (PL_strstr(aSpec, "://") == nsnull && strncmp(aSpec, "data:", 5))
+  if (PL_strstr(aSpec, "://") == nullptr && strncmp(aSpec, "data:", 5))
   {
     //XXXjag Temporary fix for bug 139362 until the real problem(bug 70083) get fixed
-    nsCAutoString uri(NS_LITERAL_CSTRING("http://"));
+    nsAutoCString uri(NS_LITERAL_CSTRING("http://"));
     uri.Append(aSpec);
-    rv = pNetService->NewURI(uri, nsnull, nsnull, aInstancePtrResult);
+    rv = pNetService->NewURI(uri, nullptr, nullptr, aInstancePtrResult);
   }
   else
-    rv = pNetService->NewURI(nsDependentCString(aSpec), nsnull, nsnull, aInstancePtrResult);
+    rv = pNetService->NewURI(nsDependentCString(aSpec), nullptr, nullptr, aInstancePtrResult);
   return rv;
 }
 
@@ -1791,18 +1773,18 @@ char
 char *
 nsMsgParseURLHost(const char *url)
 {
-  nsIURI        *workURI = nsnull;
+  nsIURI        *workURI = nullptr;
   nsresult      rv;
 
   rv = nsMsgNewURL(&workURI, url);
   if (NS_FAILED(rv) || !workURI)
-    return nsnull;
+    return nullptr;
 
-  nsCAutoString host;
+  nsAutoCString host;
   rv = workURI->GetHost(host);
   NS_IF_RELEASE(workURI);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   return ToNewCString(host);
 }
@@ -1814,15 +1796,15 @@ GenerateFileNameFromURI(nsIURI *aURL)
   nsCString file;
   nsCString spec;
   char        *returnString;
-  char        *cp = nsnull;
-  char        *cp1 = nsnull;
+  char        *cp = nullptr;
+  char        *cp1 = nullptr;
 
   rv = aURL->GetPath(file);
   if ( NS_SUCCEEDED(rv) && !file.IsEmpty())
   {
     char *newFile = ToNewCString(file);
     if (!newFile)
-      return nsnull;
+      return nullptr;
 
     // strip '/'
     cp = PL_strrchr(newFile, '/');
@@ -1844,11 +1826,11 @@ GenerateFileNameFromURI(nsIURI *aURL)
       }
     }
     else
-      return nsnull;
+      return nullptr;
   }
 
-  cp = nsnull;
-  cp1 = nsnull;
+  cp = nullptr;
+  cp1 = nullptr;
 
 
   rv = aURL->GetSpec(spec);
@@ -1856,7 +1838,7 @@ GenerateFileNameFromURI(nsIURI *aURL)
   {
     char *newSpec = ToNewCString(spec);
     if (!newSpec)
-      return nsnull;
+      return nullptr;
 
     char *cp2 = NULL, *cp3=NULL ;
 
@@ -1884,7 +1866,7 @@ GenerateFileNameFromURI(nsIURI *aURL)
     return returnString;
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 //
@@ -1897,12 +1879,12 @@ GenerateFileNameFromURI(nsIURI *aURL)
 //      Content-ID: <part1.36DF1DCE.73B5A330@netscape.com>
 //
 char *
-mime_gen_content_id(PRUint32 aPartNum, const char *aEmailAddress)
+mime_gen_content_id(uint32_t aPartNum, const char *aEmailAddress)
 {
-  PRInt32           randLen = 5;
+  int32_t           randLen = 5;
   unsigned char     rand_buf1[5];
   unsigned char     rand_buf2[5];
-  const char        *domain = nsnull;
+  const char        *domain = nullptr;
   const char        *defaultDomain = "@netscape.com";
 
   memset(rand_buf1, 0, randLen-1);
