@@ -4,6 +4,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "amIAddonManager.h"
 #include "nsWindowMemoryReporter.h"
 #include "nsGlobalWindow.h"
 #include "nsIDocument.h"
@@ -56,9 +57,9 @@ static already_AddRefed<nsIURI>
 GetWindowURI(nsIDOMWindow *aWindow)
 {
   nsCOMPtr<nsPIDOMWindow> pWindow = do_QueryInterface(aWindow);
-  NS_ENSURE_TRUE(pWindow, NULL);
+  NS_ENSURE_TRUE(pWindow, nullptr);
 
-  nsCOMPtr<nsIDocument> doc = do_QueryInterface(pWindow->GetExtantDocument());
+  nsCOMPtr<nsIDocument> doc = pWindow->GetExtantDoc();
   nsCOMPtr<nsIURI> uri;
 
   if (doc) {
@@ -68,7 +69,7 @@ GetWindowURI(nsIDOMWindow *aWindow)
   if (!uri) {
     nsCOMPtr<nsIScriptObjectPrincipal> scriptObjPrincipal =
       do_QueryInterface(aWindow);
-    NS_ENSURE_TRUE(scriptObjPrincipal, NULL);
+    NS_ENSURE_TRUE(scriptObjPrincipal, nullptr);
 
     // GetPrincipal() will print a warning if the window does not have an outer
     // window, so check here for an outer window first.  This code is
@@ -114,6 +115,7 @@ typedef nsDataHashtable<nsUint64HashKey, nsCString> WindowPaths;
 
 static nsresult
 CollectWindowReports(nsGlobalWindow *aWindow,
+                     amIAddonManager *addonManager,
                      nsWindowSizes *aWindowTotalSizes,
                      nsTHashtable<nsUint64HashKey> *aGhostWindowIDs,
                      WindowPaths *aWindowPaths,
@@ -121,16 +123,34 @@ CollectWindowReports(nsGlobalWindow *aWindow,
                      nsIMemoryMultiReporterCallback *aCb,
                      nsISupports *aClosure)
 {
-  nsAutoCString windowPath("explicit/window-objects/");
+  nsAutoCString windowPath("explicit/");
 
   // Avoid calling aWindow->GetTop() if there's no outer window.  It will work
   // just fine, but will spew a lot of warnings.
   nsGlobalWindow *top = NULL;
+  nsCOMPtr<nsIURI> location;
   if (aWindow->GetOuterWindow()) {
     // Our window should have a null top iff it has a null docshell.
     MOZ_ASSERT(!!aWindow->GetTop() == !!aWindow->GetDocShell());
     top = aWindow->GetTop();
+    if (top) {
+      location = GetWindowURI(top);
+    }
   }
+  if (!location) {
+    location = GetWindowURI(aWindow);
+  }
+
+  if (addonManager && location) {
+    bool ok;
+    nsAutoCString id;
+    if (NS_SUCCEEDED(addonManager->MapURIToAddonID(location, id, &ok)) && ok) {
+      windowPath += NS_LITERAL_CSTRING("add-ons/") + id +
+                    NS_LITERAL_CSTRING("/");
+    }
+  }
+
+  windowPath += NS_LITERAL_CSTRING("window-objects/");
 
   if (top) {
     windowPath += NS_LITERAL_CSTRING("top(");
@@ -323,10 +343,13 @@ nsWindowMemoryReporter::CollectReports(nsIMemoryMultiReporterCallback* aCb,
 
   // Collect window memory usage.
   nsWindowSizes windowTotalSizes(NULL);
+  nsCOMPtr<amIAddonManager> addonManager =
+    do_GetService("@mozilla.org/addons/integration;1");
   for (uint32_t i = 0; i < windows.Length(); i++) {
-    nsresult rv = CollectWindowReports(windows[i], &windowTotalSizes,
-                                       &ghostWindows, &windowPaths, &topWindowPaths,
-                                       aCb, aClosure);
+    nsresult rv = CollectWindowReports(windows[i], addonManager,
+                                       &windowTotalSizes, &ghostWindows,
+                                       &windowPaths, &topWindowPaths, aCb,
+                                       aClosure);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -410,14 +433,6 @@ nsWindowMemoryReporter::CollectReports(nsIMemoryMultiReporterCallback* aCb,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsWindowMemoryReporter::GetExplicitNonHeap(int64_t* aAmount)
-{
-  // This reporter only measures heap memory, so we don't need to report any
-  // bytes for it.  However, the JS multi-reporter needs to be invoked.
-  return xpc::JSMemoryMultiReporter::GetExplicitNonHeap(aAmount);
-}
-
 uint32_t
 nsWindowMemoryReporter::GetGhostTimeout()
 {
@@ -668,14 +683,6 @@ GhostURLsReporter::GetName(nsACString& aName)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsWindowMemoryReporter::
-GhostURLsReporter::GetExplicitNonHeap(int64_t* aOut)
-{
-  *aOut = 0;
-  return NS_OK;
-}
-
 struct ReportGhostWindowsEnumeratorData
 {
   nsIMemoryMultiReporterCallback* callback;
