@@ -7,9 +7,13 @@ const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/AddonManager.jsm");
+Components.utils.import("resource://gre/modules/LoginManagerContent.jsm");
 Components.utils.import("resource:///modules/Sanitizer.jsm");
 Components.utils.import("resource:///modules/mailnewsMigrator.js");
 
+var onContentLoaded = LoginManagerContent.onContentLoaded.bind(LoginManagerContent);
+var onUsernameInput = LoginManagerContent.onUsernameInput.bind(LoginManagerContent);
+
 XPCOMUtils.defineLazyModuleGetter(this, "NetUtil",
                                   "resource://gre/modules/NetUtil.jsm");
 
@@ -22,9 +26,15 @@ XPCOMUtils.defineLazyModuleGetter(this, "FileUtils",
 XPCOMUtils.defineLazyModuleGetter(this, "PlacesUtils",
                                   "resource://gre/modules/PlacesUtils.jsm");
 
+XPCOMUtils.defineLazyModuleGetter(this, "PlacesBackups",
+                                  "resource://gre/modules/PlacesBackups.jsm");
+
 XPCOMUtils.defineLazyModuleGetter(this, "BookmarkHTMLUtils",
                                   "resource://gre/modules/BookmarkHTMLUtils.jsm");
 
+XPCOMUtils.defineLazyModuleGetter(this, "DebuggerServer",
+                                  "resource://gre/modules/devtools/dbg-server.jsm");
+
 // We try to backup bookmarks at idle times, to avoid doing that at shutdown.
 // Number of idle seconds before trying to backup bookmarks.  15 minutes.
 const BOOKMARKS_BACKUP_IDLE_TIME = 15 * 60;
@@ -32,6 +42,9 @@ const BOOKMARKS_BACKUP_IDLE_TIME = 15 * 60;
 const BOOKMARKS_BACKUP_INTERVAL = 86400 * 1000;
 // Maximum number of backups to create.  Old ones will be purged.
 const BOOKMARKS_BACKUP_MAX_BACKUPS = 10;
+// Devtools Preferences
+const DEBUGGER_REMOTE_ENABLED = "devtools.debugger.remote-enabled";
+const DEBUGGER_REMOTE_PORT = "devtools.debugger.remote-port";
 
 // Constructor
 
@@ -47,9 +60,6 @@ SuiteGlue.prototype = {
   _saveSession: false,
   _sound: null,
   _isIdleObserver: false,
-  _isPlacesInitObserver: false,
-  _isPlacesLockedObserver: false,
-  _isPlacesShutdownObserver: false,
   _isPlacesDatabaseLocked: false,
   _migrationImportsDefaultBookmarks: false,
 
@@ -106,8 +116,26 @@ SuiteGlue.prototype = {
   observe: function(subject, topic, data)
   {
     switch(topic) {
-      case "xpcom-shutdown":
-        this._dispose();
+      case "nsPref:changed":
+        switch (data) {
+          case DEBUGGER_REMOTE_ENABLED:
+            if (this.dbgIsEnabled)
+              this.dbgStart();
+            else
+              this.dbgStop();
+            break;
+          case DEBUGGER_REMOTE_PORT:
+            /**
+             * If the server is not on, port changes have nothing to affect.
+             * The new value will be picked up if the server is started.
+             */
+            if (this.dbgIsEnabled)
+              this.dbgRestart();
+            break;
+        }
+        break;
+      case "profile-before-change":
+        this._onProfileShutdown();
         break;
       case "profile-after-change":
         this._onProfileAfterChange();
@@ -167,25 +195,19 @@ SuiteGlue.prototype = {
           this._initPlaces(false);
 
         Services.obs.removeObserver(this, "places-init-complete");
-        this._isPlacesInitObserver = false;
         // No longer needed, since history was initialized completely.
         Services.obs.removeObserver(this, "places-database-locked");
-        this._isPlacesLockedObserver = false;
         break;
       case "places-database-locked":
         this._isPlacesDatabaseLocked = true;
         // Stop observing, so further attempts to load history service
         // will not show the prompt.
         Services.obs.removeObserver(this, "places-database-locked");
-        this._isPlacesLockedObserver = false;
         break;
       case "places-shutdown":
-        if (this._isPlacesShutdownObserver) {
-          Services.obs.removeObserver(this, "places-shutdown");
-          this._isPlacesShutdownObserver = false;
-        }
+        Services.obs.removeObserver(this, "places-shutdown");
         // places-shutdown is fired when the profile is about to disappear.
-        this._onProfileShutdown();
+        this._onPlacesShutdown();
         break;
       case "idle":
         if (this._idleService.idleTime > BOOKMARKS_BACKUP_IDLE_TIME * 1000)
@@ -194,16 +216,32 @@ SuiteGlue.prototype = {
       case "initial-migration":
         this._initialMigrationPerformed = true;
         break;
+      case "browser-search-engine-modified":
+        if (data != "engine-default" && data != "engine-current") {
+          break;
+        }
+        // Enforce that the search service's defaultEngine is always equal to
+        // its currentEngine. The search service will notify us any time either
+        // of them are changed (either by directly setting the relevant prefs,
+        // i.e. if add-ons try to change this directly, or if the
+        // nsIBrowserSearchService setters are called).
+        var ss = Services.search;
+        if (ss.currentEngine.name == ss.defaultEngine.name)
+          return;
+        if (data == "engine-current")
+          ss.defaultEngine = ss.currentEngine;
+        else
+          ss.currentEngine = ss.defaultEngine;
+        break;
     }
   },
 
   // nsIWebProgressListener partial implementation
   onLocationChange: function(aWebProgress, aRequest, aLocation, aFlags)
   {
-    if (aWebProgress.DOMWindow.top == aWebProgress.DOMWindow &&
+    if (aWebProgress.isTopLevel &&
         aWebProgress instanceof Components.interfaces.nsIDocShell &&
         aWebProgress.loadType & Components.interfaces.nsIDocShell.LOAD_CMD_NORMAL &&
-        aWebProgress instanceof Components.interfaces.nsIDocShellHistory &&
         aWebProgress.useGlobalHistory &&
         aWebProgress instanceof Components.interfaces.nsILoadContext &&
         !aWebProgress.usePrivateBrowsing) {
@@ -231,62 +269,38 @@ SuiteGlue.prototype = {
     }
   },
 
+  onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
+    aWebProgress.DOMWindow.addEventListener("DOMContentLoaded", onContentLoaded, true);
+    aWebProgress.DOMWindow.addEventListener("DOMAutoComplete", onUsernameInput, true);
+    aWebProgress.DOMWindow.addEventListener("change", onUsernameInput, true);
+  },
+
   // initialization (called on application startup)
   _init: function()
   {
     // observer registration
-    Services.obs.addObserver(this, "xpcom-shutdown", false);
-    Services.obs.addObserver(this, "profile-after-change", false);
-    Services.obs.addObserver(this, "final-ui-startup", false);
-    Services.obs.addObserver(this, "sessionstore-windows-restored", false);
-    Services.obs.addObserver(this, "browser:purge-session-history", false);
-    Services.obs.addObserver(this, "quit-application-requested", false);
-    Services.obs.addObserver(this, "quit-application-granted", false);
-    Services.obs.addObserver(this, "browser-lastwindow-close-requested", false);
-    Services.obs.addObserver(this, "browser-lastwindow-close-granted", false);
-    Services.obs.addObserver(this, "console-api-log-event", false);
-    Services.obs.addObserver(this, "weave:service:ready", false);
-    Services.obs.addObserver(this, "weave:engine:clients:display-uri", false);
-    Services.obs.addObserver(this, "session-save", false);
-    Services.obs.addObserver(this, "dl-done", false);
-    Services.obs.addObserver(this, "places-init-complete", false);
-    this._isPlacesInitObserver = true;
-    Services.obs.addObserver(this, "places-database-locked", false);
-    this._isPlacesLockedObserver = true;
-    Services.obs.addObserver(this, "places-shutdown", false);
-    this._isPlacesShutdownObserver = true;
+    Services.obs.addObserver(this, "profile-before-change", true);
+    Services.obs.addObserver(this, "profile-after-change", true);
+    Services.obs.addObserver(this, "final-ui-startup", true);
+    Services.obs.addObserver(this, "sessionstore-windows-restored", true);
+    Services.obs.addObserver(this, "browser:purge-session-history", true);
+    Services.obs.addObserver(this, "quit-application-requested", true);
+    Services.obs.addObserver(this, "quit-application-granted", true);
+    Services.obs.addObserver(this, "browser-lastwindow-close-requested", true);
+    Services.obs.addObserver(this, "browser-lastwindow-close-granted", true);
+    Services.obs.addObserver(this, "console-api-log-event", true);
+    Services.obs.addObserver(this, "weave:service:ready", true);
+    Services.obs.addObserver(this, "weave:engine:clients:display-uri", true);
+    Services.obs.addObserver(this, "session-save", true);
+    Services.obs.addObserver(this, "dl-done", true);
+    Services.obs.addObserver(this, "places-init-complete", true);
+    Services.obs.addObserver(this, "places-database-locked", true);
+    Services.obs.addObserver(this, "places-shutdown", true);
+    Services.obs.addObserver(this, "browser-search-engine-modified", true);
+    Services.prefs.addObserver("devtools.debugger.", this, true);
     Components.classes['@mozilla.org/docloaderservice;1']
               .getService(Components.interfaces.nsIWebProgress)
-              .addProgressListener(this, Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
-  },
-
-  // cleanup (called on application shutdown)
-  _dispose: function()
-  {
-    // observer removal
-    Services.obs.removeObserver(this, "xpcom-shutdown");
-    Services.obs.removeObserver(this, "profile-after-change");
-    Services.obs.removeObserver(this, "final-ui-startup");
-    Services.obs.removeObserver(this, "sessionstore-windows-restored");
-    Services.obs.removeObserver(this, "browser:purge-session-history");
-    Services.obs.removeObserver(this, "quit-application-requested");
-    Services.obs.removeObserver(this, "quit-application-granted");
-    Services.obs.removeObserver(this, "browser-lastwindow-close-requested");
-    Services.obs.removeObserver(this, "browser-lastwindow-close-granted");
-    Services.obs.removeObserver(this, "console-api-log-event");
-    Services.obs.removeObserver(this, "weave:service:ready");
-    Services.obs.removeObserver(this, "weave:engine:clients:display-uri");
-    Services.obs.removeObserver(this, "session-save");
-    Services.obs.removeObserver(this, "dl-done");
-    if (this._isIdleObserver)
-      this._idleService.removeIdleObserver(this, BOOKMARKS_BACKUP_IDLE_TIME);
-    if (this._isPlacesInitObserver)
-      Services.obs.removeObserver(this, "places-init-complete");
-    if (this._isPlacesLockedObserver)
-      Services.obs.removeObserver(this, "places-database-locked");
-    if (this._isPlacesShutdownObserver)
-      Services.obs.removeObserver(this, "places-shutdown");
-    UserAgentOverrides.uninit();
+              .addProgressListener(this, Components.interfaces.nsIWebProgress.NOTIFY_LOCATION | Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
   },
 
   // profile is available
@@ -386,12 +400,14 @@ SuiteGlue.prototype = {
     this._checkForDefaultClient(aWindow);
   },
 
-  // profile shutdown handler (contains profile cleanup routines)
+  /**
+   * Profile shutdown handler (contains profile cleanup routines).
+   * All components depending on Places should be shut down in
+   * _onPlacesShutdown() and not here.
+   */
   _onProfileShutdown: function()
   {
-    this._shutdownPlaces();
-    if (!Sanitizer.doPendingSanitize())
-      Services.prefs.setBoolPref("privacy.sanitize.didShutdownSanitize", true);
+    UserAgentOverrides.uninit()
   },
 
   _promptForMasterPassword: function()
@@ -688,7 +704,7 @@ SuiteGlue.prototype = {
     // We must instantiate the history service since it will tell us if we
     // need to import or restore bookmarks due to first-run, corruption or
     // forced migration (due to a major schema change).
-    var bookmarksBackupFile = PlacesUtils.backups.getMostRecent("json");
+    var bookmarksBackupFile = PlacesBackups.getMostRecent("json");
 
     // If the database is corrupt or has been newly created we should
     // import bookmarks. Same if we don't have any JSON backups, which
@@ -824,11 +840,9 @@ SuiteGlue.prototype = {
    * Places shut-down tasks
    * - back up bookmarks if needed.
    * - export bookmarks as HTML, if so configured.
-   *
-   * Note: quit-application-granted notification is received twice
-   *       so replace this method with a no-op when first called.
+   * - finalize components depending on Places.
    */
-  _shutdownPlaces: function() {
+  _onPlacesShutdown: function() {
     if (this._isIdleObserver) {
       this._idleService.removeIdleObserver(this, BOOKMARKS_BACKUP_IDLE_TIME);
       this._isIdleObserver = false;
@@ -862,24 +876,27 @@ SuiteGlue.prototype = {
         }
       }
     } catch(ex) { /* Don't export */ }
+
+    if (!Sanitizer.doPendingSanitize())
+      Services.prefs.setBoolPref("privacy.sanitize.didShutdownSanitize", true);
   },
 
   /**
    * Backup bookmarks if needed.
    */
   _backupBookmarks: function() {
-    let lastBackupFile = PlacesUtils.backups.getMostRecent();
+    let lastBackupFile = PlacesBackups.getMostRecent();
 
     // Backup bookmarks if there are no backups or the maximum interval between
     // backups elapsed.
     if (!lastBackupFile ||
-        new Date() - PlacesUtils.backups.getDateForFile(lastBackupFile) > BOOKMARKS_BACKUP_INTERVAL) {
+        new Date() - PlacesBackups.getDateForFile(lastBackupFile) > BOOKMARKS_BACKUP_INTERVAL) {
       let maxBackups = BOOKMARKS_BACKUP_MAX_BACKUPS;
       try {
         maxBackups = Services.prefs.getIntPref("browser.bookmarks.max_backups");
       } catch(ex) { /* Use default. */ }
 
-      PlacesUtils.backups.create(maxBackups); // Don't force creation.
+      PlacesBackups.create(maxBackups); // Don't force creation.
     }
   },
 
@@ -918,6 +935,36 @@ SuiteGlue.prototype = {
     } catch (e) {}
   },
 
+  /**
+   * Devtools Debugger
+   */
+  get dbgIsEnabled()
+  {
+    return Services.prefs.getBoolPref(DEBUGGER_REMOTE_ENABLED);
+  },
+
+  dbgStart: function()
+  {
+    var port = Services.prefs.getIntPref(DEBUGGER_REMOTE_PORT);
+    if (!DebuggerServer.initialized) {
+      DebuggerServer.init();
+      DebuggerServer.addBrowserActors();
+    }
+    DebuggerServer.openListener(port);
+  },
+
+  dbgStop: function()
+  {
+    if (DebuggerServer.initialized)
+      DebuggerServer.closeListener();
+  },
+
+  dbgRestart: function()
+  {
+    this.dbgStop();
+    this.dbgStart();
+  },
+
   // ------------------------------
   // public nsISuiteGlue members
   // ------------------------------
@@ -1118,7 +1165,12 @@ ContentPermissionPrompt.prototype = {
 
   prompt: function(aRequest)
   {
-    if (aRequest.type != "geolocation")
+    const kFeatureKeys = { "geolocation" : "geo",
+                           "desktop-notification" : "desktop-notification",
+                         };
+
+    // Make sure that we support the request.
+    if (!(aRequest.type in kFeatureKeys))
       return;
 
     var path, host;
@@ -1133,7 +1185,8 @@ ContentPermissionPrompt.prototype = {
     else
       return;
 
-    switch (Services.perms.testExactPermissionFromPrincipal(requestingPrincipal, "geo")) {
+    var perm = kFeatureKeys[aRequest.type];
+    switch (Services.perms.testExactPermissionFromPrincipal(requestingPrincipal, perm)) {
       case Services.perms.ALLOW_ACTION:
         aRequest.allow();
         return;
@@ -1142,26 +1195,38 @@ ContentPermissionPrompt.prototype = {
         return;
     }
 
-    function allowCallback(remember) {
+    function allowCallback(remember, expireType) {
       if (remember)
-        Services.perms.addFromPrincipal(requestingPrincipal, "geo", Services.perms.ALLOW_ACTION);
+        Services.perms.addFromPrincipal(requestingPrincipal, perm,
+                                        Services.perms.ALLOW_ACTION,
+                                        expireType);
       aRequest.allow();
     }
 
-    function cancelCallback(remember) {
+    function cancelCallback(remember, expireType) {
       if (remember)
-        Services.perms.addFromPrincipal(requestingPrincipal, "geo", Services.perms.DENY_ACTION);
+        Services.perms.addFromPrincipal(requestingPrincipal, perm,
+                                        Services.perms.DENY_ACTION,
+                                        expireType);
       aRequest.cancel();
     }
 
-    aRequest.window
-            .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-            .getInterface(Components.interfaces.nsIWebNavigation)
-            .QueryInterface(Components.interfaces.nsIDocShell)
-            .chromeEventHandler.parentNode.wrappedJSObject
-            .showGeolocationPrompt(path, host,
-                                   allowCallback,
-                                   cancelCallback);
+    var nb = aRequest.window
+                     .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+                     .getInterface(Components.interfaces.nsIWebNavigation)
+                     .QueryInterface(Components.interfaces.nsIDocShell)
+                     .chromeEventHandler.parentNode;
+
+    // Show the prompt.
+    switch (aRequest.type) {
+      case "geolocation":
+        nb.showGeolocationPrompt(path, host, allowCallback, cancelCallback);
+        break;
+      case "desktop-notification":
+        if (host)
+          nb.showWebNotificationPrompt(host, allowCallback, cancelCallback);
+        break;
+    }
   },
 };
 
